<?php
namespace RCP\Addon\WooCommerceMemberDiscounts;

class Discounts {

	/**
	 * ID of the current user.
	 *
	 * @var int
	 */
	private $user_id;

	/**
	 * Customer object
	 *
	 * @var \RCP_Customer
	 */
	private $customer;

	/**
	 * ID of the user's membership level.
	 *
	 * @var int
	 * @deprecated 1.0.4 In favour of `$membership_level_ids`.
	 */
	private $sub_id;

	/**
	 * Array of membership level IDs the current user has active memberships for.
	 *
	 * @var array
	 */
	private $membership_level_ids = array();

	/**
	 * Array of coupon code IDs that are valid for all the user's membership levels. These are the coupons that
	 * will potentially be auto added.
	 *
	 * @var array
	 */
	private $coupons = array();

	/**
	 * Whether or not the coupon is valid for members only.
	 *
	 * @var bool
	 */
	private $members_only;

	/**
	 * Discounts constructor.
	 */
	public function __construct() {
		$this->user_id = wp_get_current_user()->ID;

		if ( function_exists( 'rcp_get_customer_by_user_id' ) ) {
			$this->customer = rcp_get_customer_by_user_id( $this->user_id );

			if ( ! empty( $this->customer ) ) {
				$memberships = $this->customer->get_memberships( array(
					'status' => array( 'active', 'cancelled' )
				) );

				if ( ! empty( $memberships ) ) {
					foreach ( $memberships as $membership ) {
						/**
						 * @var \RCP_Membership $membership
						 */
						if ( $membership->is_active() ) {
							$this->membership_level_ids[] = $membership->get_object_id();
						}
					}

					$this->membership_level_ids = array_unique( $this->membership_level_ids );
				}
			}

			// Backwards compat.
			$this->sub_id = ! empty( $this->membership_level_ids[0] ) ? $this->membership_level_ids[0] : 0;
		} else {
			$this->sub_id  = (int) rcp_get_subscription_id( $this->user_id );
			$this->membership_level_ids = array( $this->sub_id );
		}
	}

	/**
	 * Adds the plugin hooks.
	 */
	public function hooks() {
		add_action( 'template_redirect', array( $this, 'apply_coupon' ) );
		add_filter( 'woocommerce_coupon_is_valid', array( $this, 'validate_members_only_coupon' ), 10, 2 );
	}

	/**
	 * Applies applicable coupons on the checkout page.
	 */
	public function apply_coupon() {

		if ( empty( $this->user_id ) || empty( $this->membership_level_ids ) || ( ! is_checkout() && ! is_cart() ) ) {
			return;
		}

		/**
		 * @var \RCP_Levels $rcp_levels_db
		 */
		global $rcp_levels_db;

		foreach ( $this->membership_level_ids as $membership_level_id ) {
			$level_coupons = $rcp_levels_db->get_meta( $membership_level_id, 'member_discount_coupon_woo', false );

			if ( ! empty( $level_coupons ) ) {
				$this->coupons = array_merge( $this->coupons, $level_coupons );
			}
		}

		if ( empty( $this->coupons ) ) {
			return;
		}

		$this->coupons = array_unique( $this->coupons );

		foreach ( $this->coupons as $key => $coupon ) {

			$coupon_object = get_post( $coupon );

			if ( ! is_object( $coupon_object ) || ! is_a( $coupon_object, 'WP_Post' ) ) {
				return;
			}

			$wc_coupon    = new \WC_Coupon( $coupon_object->post_title );
			$discounts    = new \WC_Discounts( WC()->cart );
			$valid_coupon = false;

			if ( version_compare( WC_VERSION, '3.2', '>=' ) ) {
				if ( true === $discounts->is_coupon_valid( $wc_coupon ) ) {
					$valid_coupon = true;
				}
			} else {
				$valid_coupon = $wc_coupon->is_valid();
			}

			if ( ! WC()->cart->has_discount( $coupon_object->post_title ) && $valid_coupon ) {
				WC()->cart->add_discount( $coupon_object->post_title );
			}

		}
	}

	/**
	 * Validates members only coupons.
	 *
	 * @param bool $valid
	 * @param \WC_Coupon $obj
	 *
	 * @return bool
	 */
	public function validate_members_only_coupon( $valid, $obj ) {

		$this->members_only = get_post_meta( $obj->get_id(), 'rcp_woo_member_discount_coupon_active_members_only', true );

		// If not a member's only coupon, return early.
		if ( empty( $this->members_only ) ) {
			return $valid;
		}

		if ( version_compare( RCP_PLUGIN_VERSION, '3.0', '>=' ) ) {

			// User is not logged in or not a customer - bail.
			if ( empty( $this->customer ) ) {
				return false;
			}

			// User does not have any active memberships - bail.
			if ( ! $this->customer->has_active_membership() ) {
				return false;
			}

		} else {

			// If user not logged in or is an expired member, coupon is not valid.
			if ( ! is_user_logged_in() || empty( $this->sub_id ) || rcp_is_expired( $this->user_id ) ) {
				return false;
			}

		}

		return $valid;
	}
}